// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"

#define TIMESLOTS 30
float oldtime[TIMESLOTS] = {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f};
int timepos = 0;

void writefps() {
	HDC dc;

	LARGE_INTEGER clocks, freq;

	QueryPerformanceCounter(&clocks);
	QueryPerformanceFrequency(&freq);

	oldtime[timepos] = float(clocks.QuadPart) / freq.QuadPart;

	float frametime = oldtime[timepos] - oldtime[(timepos+1) % TIMESLOTS];

	timepos++;
	timepos = timepos % TIMESLOTS;

	DX.DDSRender->GetDC(&dc);

	char str[256];
	sprintf(str, "%.2f", float(TIMESLOTS)/frametime);

	TextOut(dc,0,0,str,strlen(str));
	
	DX.DDSRender->ReleaseDC(dc);
}


/* Important */
FX_ENTRY void FX_CALL grBufferSwap( int swap_interval ) {
	vlog("Called grBufferSwap");

	_exebuffer->Flush();

	if (Settings.fps) writefps();

	DX.D3DD->EndScene();
	if (Settings.vsync) {
		DX.DDSPrimary->Flip(0, DDFLIP_WAIT); 
	} else {
		DX.DDSPrimary->Flip(0, DDFLIP_NOVSYNC ); 
	}
	DX.D3DD->BeginScene();

}




/* Only for Quake 2 */
FX_ENTRY void FX_CALL grRenderBuffer(GrBuffer_t buffer) {
	vlog("Called grRenderBuffer");

//	_exebuffer->Flush();

//	log("  Error, unsupported function: grRenderBuffer");

}


/* Important */
FX_ENTRY void FX_CALL grDepthBufferFunction( GrCmpFnc_t function ) {
	vlog("Called grDepthBufferFunction: function=" << function);

	_exebuffer->Flush();

	switch(function) {
	case GR_CMP_NEVER:		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_NEVER);		break;
	case GR_CMP_LESS:		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_LESS);			break;
	case GR_CMP_EQUAL:		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_EQUAL);		break;
	case GR_CMP_LEQUAL:		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_LESSEQUAL);	break;
	case GR_CMP_GREATER:	DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_GREATER);		break;
	case GR_CMP_NOTEQUAL:	DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_NOTEQUAL);		break;
	case GR_CMP_GEQUAL:		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_GREATEREQUAL);	break;
	case GR_CMP_ALWAYS:		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_ALWAYS);		break;
	}
//	DX.D3DD->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_ALWAYS);
}




/* Only for Marble32 */
FX_ENTRY void FX_CALL grClipWindow(FxU32 minx, FxU32 miny, FxU32 maxx, FxU32 maxy ) {
	vlog("Called grClipWindow: left=" << minx << "  right=" << maxx << "  top=" << miny << "  bottom=" << maxy);

	_exebuffer->Flush();

	Voodoo.viewport.dwX = minx;
	Voodoo.viewport.dwWidth = maxx - minx;
	Voodoo.viewport.dwHeight = maxy - miny;

	if (Voodoo.origin == GR_ORIGIN_UPPER_LEFT) {
		Voodoo.viewport.dwY = miny;
	} else {
		Voodoo.viewport.dwY = Voodoo.height - maxy - 1;
	}

	DX.D3DViewport->SetViewport2(&Voodoo.viewport);
//	if (DX.D3DViewport->SetViewport2(&Voodoo.viewport) != D3D_OK) log("  Error in SetViewport2 in grClipWindow: (" << minx << ", " << maxx << ", " << miny << ", " << maxy << ")");

}




FX_ENTRY void FX_CALL grDepthBufferMode( GrDepthBufferMode_t mode ) {
	vlog("Called grDepthBufferMode: mode=" << mode);

	_exebuffer->Flush();

	Voodoo.dbMode = mode;

	switch (Voodoo.dbMode) {
	case GR_DEPTHBUFFER_DISABLE:
		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_FALSE);
		break;
	case GR_DEPTHBUFFER_ZBUFFER:
	case GR_DEPTHBUFFER_ZBUFFER_COMPARE_TO_BIAS:
		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_TRUE);
		break;
	case GR_DEPTHBUFFER_WBUFFER:
	case GR_DEPTHBUFFER_WBUFFER_COMPARE_TO_BIAS:
		if (DX.D3DHD.dpcTriCaps.dwRasterCaps & D3DPRASTERCAPS_WBUFFER) {
			DX.D3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_USEW);
		} else {
			DX.D3DD->SetRenderState(D3DRENDERSTATE_ZENABLE, D3DZB_TRUE);
		}
		break;
	}

	Voodoo.dbMode = mode;
	vconv = vconv_lut[depthbuf_lut[Voodoo.dbMode]][Voodoo.origin][Voodoo.texture];
}


FX_ENTRY void FX_CALL grBufferClear( GrColor_t color, GrAlpha_t alpha, FxU16 depth ) {
	vlog("Called grBufferClear");

	_exebuffer->Flush();

	D3DRECT r = {0, 0, Voodoo.width, Voodoo.height};

	// Todo: convert depth buffer field
	DX.D3DViewport->Clear2(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x0000, 1.0, 0x0000);
}



FX_ENTRY int FX_CALL grBufferNumPending( void ) {
	vlog("Called grBufferNumPending");

	return 0;
}

