// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#define  STRICT
#define  D3D_OVERLOADS

//#define INITGUID

#define __WIN32__

#include <math.h>
#include <float.h>

#include <time.h>
#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>

#include <ddraw.h>
#include <d3d.h>


#include "D3DUtil.h"
#include "D3DEnum.h"
#include "D3DMath.h"

#include "glide.h"
#include "glideutl.h"

typedef unsigned char uchar;

#include "colorconv.h"



typedef FxU32 GrHints_t;

extern __int64 prof_temp;
extern __int64 func_time;

#define prof_start(v) \
	v = 0;

#define prof_enter(v) \
	__asm { _emit 0x0f } \
	__asm { _emit 0x31 } \
	__asm { mov dword ptr prof_temp, eax } \
	__asm { mov dword ptr prof_temp+4, edx } \
	v -= prof_temp;

#define prof_exit(v) \
	__asm { _emit 0x0f } \
	__asm { _emit 0x31 } \
	__asm { mov dword ptr prof_temp, eax } \
	__asm { mov dword ptr prof_temp+4, edx } \
	v += prof_temp;

#define LOG
//#define VERBOSE_LOG
//#define VERY_VERBOSE_LOG

#ifdef LOG
#define log(text) { if (Settings.logging) { of.open("glide.log", ios::app); of << text << "\n"; of.close(); } }
//#define log(text) { of << text << "\n"; of.flush(); }
#else
#define log(test) {}
#endif

#ifdef VERBOSE_LOG
#define vlog(text) { if (Settings.logging) { of.open("glide.log", ios::app); of << text << "\n"; of.close(); } }
//#define vlog(text) { of << text << "\n"; of.flush(); }
#else
#define vlog(test) {}
#endif

#ifdef VERY_VERBOSE_LOG
#define vvlog(text) { if (Settings.logging) { of.open("glide.log", ios::app); of << text << "\n"; of.close(); } }
#else
#define vvlog(test) {}
#endif

#define MAX_TEXTURES 2048
#define TEX_CACHE_SIZE 64


D3DCOLOR colorconv(GrColor_t);

D3DCOLOR iterated(const GrVertex &);
D3DCOLOR iterated2(const GrVertex &);
D3DCOLOR iterconst(const GrVertex &);
D3DCOLOR constant(const GrVertex &);
D3DCOLOR constant2(const GrVertex &);
D3DCOLOR white(const GrVertex &);
D3DCOLOR black(const GrVertex &);
extern D3DCOLOR (*getcolor)(const GrVertex &);

void vconv_D_UL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_UL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z16_UL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z24_UL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z32_UL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_Z_UL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_D_LL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_LL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z16_LL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z24_LL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z32_LL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_Z_LL_T(const GrVertex &, D3DTLVERTEX &);
void vconv_D_UL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_UL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z16_UL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z24_UL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z32_UL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_Z_UL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_D_LL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_LL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z16_LL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z24_LL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_W_Z32_LL_NT(const GrVertex &, D3DTLVERTEX &);
void vconv_Z_LL_NT(const GrVertex &, D3DTLVERTEX &);
extern void (*vconv)(const GrVertex &, D3DTLVERTEX &);

GrTextureFormat_t getTextureFormat(char *);
GrAspectRatio_t getAspectRatio(FxU32, FxU32);
FxU32 getBytesPerPixel(GrTextureFormat_t);
FxU32 getLfbWriteBytesPerPixel(GrLfbWriteMode_t);
FxU32 getMemRequired(Gu3dfInfo *, FxU32, FxU32);
FxU32 TexMemCalc(FxU32, GrLOD_t, GrLOD_t, GrAspectRatio_t, GrTextureFormat_t);
void table_init();
FxBool getcaps();
FxBool getzbuffer(DDPIXELFORMAT &);
FxBool loadsettings();

extern ofstream of;

extern HRESULT hr;
extern HINSTANCE _hInstance;


extern GrColorFormat_t			_color_format;

extern FxFloat					*smul;
extern FxFloat					*tmul;
extern FxU32					*src_blend_lut;
extern FxU32					*blend_caps_lut;
extern FxU32					*dst_blend_lut;
extern FxU32					*atest_lut;
extern FxU32					*atest_caps_lut;

extern D3DMATRIX				_idenMat;


extern GrErrorCallbackFnc_t		_errorFnc;


struct sSettings {
	DWORD colorop_add;
	DWORD colorop_blendtexturealpha;
	DWORD alphaop_modulate;
	DWORD clamp;
	DWORD aa;
	DWORD zbits;
	DWORD vsync;
	DWORD tmumem;
	DWORD model;
	DWORD backbuffers;
	DWORD fps;
	DWORD priority;
	DWORD logging;
	DWORD atifix;
};


struct sDX {
	LPDIRECTDRAW4					DD;
	LPDIRECT3D3						D3D;

	LPDIRECTDRAWSURFACE4			DDSPrimary;
	LPDIRECTDRAWSURFACE4			DDSRender;
	LPDIRECTDRAWSURFACE4			DDSZBuffer;
	D3DDEVICEDESC					D3DHD;
	D3DDEVICEDESC					D3DSD;
	HWND							hWnd;
	LPDIRECT3DDEVICE3				D3DD;

	LPDIRECT3DVIEWPORT3				D3DViewport;

	DWORD							lastColorOp;
	DWORD							lastColorArg1;
	DWORD							lastColorArg2;
	DWORD							lastAlphaOp;
	DWORD							lastAlphaArg1;
	DWORD							lastAlphaArg2;
};

class Tex;

struct sVoodoo {
	FxU32						fbRam;
	FxU32						tmuRam;
	char						*tmuMem;
	FxU32						width;
	FxU32						height;
	FxU32						refresh;
	PALETTEENTRY				palette[256];
	int							palnum;			// Unique palette identifier
	Tex							*curtex;
	D3DVIEWPORT2				viewport;

	GrOriginLocation_t			origin;
	GrDepthBufferMode_t			dbMode;
	GrColorFormat_t				colorFormat;
	FxU32						texture;
	FxU32						Zdepth;

	D3DCOLOR					constantColor;
	DWORD						colorKey;
	int							colorkeyon;

	FxFloat						CCR, CCG, CCB, CCA;

	FxU32						ccLocal;
	FxU32						ccFunc;

	int							uclamp;
	int							vclamp;

	int							openwindow;

//	int							multitexture;

//	int							lastpolytype;
};

extern sVoodoo					Voodoo;
extern sDX						DX;
extern sSettings				Settings;

#include "Tex.h"
#include "TexList.h"
#include "ExeBuffer.h"


extern TexList					*_texlist;

extern ExeBuffer				*_exebuffer;


#define DEPTH_D     0
#define DEPTH_W		1
#define DEPTH_W_Z16 2
#define DEPTH_W_Z24 3
#define DEPTH_W_Z32 4
#define DEPTH_Z		5
#define DEPTH_MAX	6

extern int depthbuf_lut[5];
extern FxU16 yiq_lut[256];

#define TEXTURE				0
#define NO_TEXTURE			1
#define TEXTURE_MODE_MAX	2

extern void (*(vconv_lut[DEPTH_MAX][2][TEXTURE_MODE_MAX]))(const GrVertex &, D3DTLVERTEX &);

extern HFONT fontInUse;
