// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"


#define fill_blend_lut(table, glidemode, d3dmode1, d3dmode2, d3dmode3, d3dmode4) \
	table[glidemode][0] = d3dmode1; \
	table[glidemode][1] = d3dmode2; \
	table[glidemode][2] = d3dmode3; \
	table[glidemode][3] = d3dmode4;

#define fill_lut(table, caps, caps_lut, glidemode, d3dmode1, d3dmode2, d3dmode3, d3dmode4) \
	if (caps & caps_lut[d3dmode1]) { \
		table[glidemode] = d3dmode1; \
	} else if (caps & caps_lut[d3dmode2]) { \
		log("Mode " #d3dmode1 " not supported in " #table "--reverting to mode " #d3dmode2); \
		table[glidemode] = d3dmode2; \
	} else if (caps & caps_lut[d3dmode3]) { \
		log("Mode " #d3dmode1 " not supported in " #table "--reverting to mode " #d3dmode3); \
		table[glidemode] = d3dmode3; \
	} else if (caps & caps_lut[d3dmode4]) { \
		log("Mode " #d3dmode1 " not supported in " #table "--reverting to mode " #d3dmode4); \
		table[glidemode] = d3dmode4; \
	} else { \
		log("No alternative blend modes found for mode " #d3dmode1 " in " #table ".  Please contact the author."); \
	}


#define fill_src_blend_lut(glidemode, d3dmode1, d3dmode2, d3dmode3, d3dmode4) \
	fill_lut(src_blend_lut, DX.D3DHD.dpcTriCaps.dwSrcBlendCaps, blend_caps_lut, glidemode, d3dmode1, d3dmode2, d3dmode3, d3dmode4)

#define fill_dst_blend_lut(glidemode, d3dmode1, d3dmode2, d3dmode3, d3dmode4) \
	fill_lut(dst_blend_lut, DX.D3DHD.dpcTriCaps.dwDestBlendCaps, blend_caps_lut, glidemode, d3dmode1, d3dmode2, d3dmode3, d3dmode4)

/* Important */
//extern void __stdcall grGlideInit( void ) {
FX_ENTRY void FX_CALL grGlideInit(void) {
	log("Called grGlideInit");

	loadsettings();

	Voodoo.tmuRam = Settings.tmumem;

	Voodoo.tmuMem = new char[Voodoo.tmuRam*1024*1024+8];
	Voodoo.tmuMem += (DWORD)Voodoo.tmuMem % 8;

	Voodoo.fbRam = 2;
	Voodoo.colorkeyon = 0;
	Voodoo.palnum = -1;

	_texlist = new TexList();
	_exebuffer = new ExeBuffer();


    D3DUtil_SetIdentityMatrix( _idenMat );

	// Make sure initial values aren't real ones...
	DX.lastColorOp = 0xFFFFFFFF;
	DX.lastColorArg1 = 0xFFFFFFFF;
	DX.lastColorArg2 = 0xFFFFFFFF;
	DX.lastAlphaOp = 0xFFFFFFFF;
	DX.lastAlphaArg1 = 0xFFFFFFFF;
	DX.lastAlphaArg2 = 0xFFFFFFFF;

}

/*
static BOOL CALLBACK enumProc(HWND hwnd, LPARAM lParam)
{
	_hWnd = hwnd;

	return TRUE;
}
*/


/* Maybe important */
FX_ENTRY void FX_CALL grGlideGetVersion (char version[80]) {
	log("Called grGlideGetVersion");

	strcpy(version, "Scott Cutler's Direct3D to Glide Wrapper v0.01");
}



/* Important */
FX_ENTRY void FX_CALL grGlideShutdown( void ) {
	log("Called grGlideShutdown");

	grSstWinClose();

	delete [] Voodoo.tmuMem;

}


/* Only for Marble32 */
FX_ENTRY void FX_CALL grGlideGetState( GrState *state ) {
	vlog("Called grGlideGetState");

}

/* Only for Marble32 */
FX_ENTRY void FX_CALL grGlideSetState(const GrState *state) {
	vlog("Called grGlideSetState");

}


void table_init() {

	// Init texture scaling vars for different aspect ratios
	smul = new FxFloat[21];  // More than enough, just to be safe
	tmul = new FxFloat[21];

	smul += 11;		// I can't tell for sure, but it looks like in some cases the
	tmul += 11;		// aspect ratio indexes could be negative.  Doing this keeps this
					// from having an access violation.

	smul[GR_ASPECT_8x1] = 1.0f / 256.0f;
	smul[GR_ASPECT_4x1] = 1.0f / 256.0f;
	smul[GR_ASPECT_2x1] = 1.0f / 256.0f;
	smul[GR_ASPECT_1x1] = 1.0f / 256.0f;
	smul[GR_ASPECT_1x2] = 1.0f / 128.0f;
	smul[GR_ASPECT_1x4] = 1.0f / 64.0f;
	smul[GR_ASPECT_1x8] = 1.0f / 32.0f;
	tmul[GR_ASPECT_8x1] = 1.0f / 32.0f;
	tmul[GR_ASPECT_4x1] = 1.0f / 64.0f;
	tmul[GR_ASPECT_2x1] = 1.0f / 128.0f;
	tmul[GR_ASPECT_1x1] = 1.0f / 256.0f;
	tmul[GR_ASPECT_1x2] = 1.0f / 256.0f;
	tmul[GR_ASPECT_1x4] = 1.0f / 256.0f;
	tmul[GR_ASPECT_1x8] = 1.0f / 256.0f;



	src_blend_lut = new FxU32[16];
	blend_caps_lut = new FxU32[16];
	dst_blend_lut = new FxU32[16];


	blend_caps_lut[D3DBLEND_ZERO] =				D3DPBLENDCAPS_ZERO;
	blend_caps_lut[D3DBLEND_ONE] =				D3DPBLENDCAPS_ONE;
	blend_caps_lut[D3DBLEND_SRCCOLOR] =			D3DPBLENDCAPS_SRCCOLOR;
	blend_caps_lut[D3DBLEND_INVSRCCOLOR] =		D3DPBLENDCAPS_INVSRCCOLOR;
	blend_caps_lut[D3DBLEND_SRCALPHA] =			D3DPBLENDCAPS_SRCALPHA;
	blend_caps_lut[D3DBLEND_INVSRCALPHA] =		D3DPBLENDCAPS_INVSRCALPHA;
	blend_caps_lut[D3DBLEND_DESTALPHA] =		D3DPBLENDCAPS_DESTALPHA;
	blend_caps_lut[D3DBLEND_INVDESTALPHA] =		D3DPBLENDCAPS_INVDESTALPHA;
	blend_caps_lut[D3DBLEND_DESTCOLOR] =		D3DPBLENDCAPS_DESTCOLOR;
	blend_caps_lut[D3DBLEND_INVDESTCOLOR] =		D3DPBLENDCAPS_INVDESTCOLOR;
	blend_caps_lut[D3DBLEND_SRCALPHASAT] =		D3DPBLENDCAPS_SRCALPHASAT;
	blend_caps_lut[D3DBLEND_BOTHSRCALPHA] =		D3DPBLENDCAPS_BOTHSRCALPHA;
	blend_caps_lut[D3DBLEND_BOTHINVSRCALPHA] =	D3DPBLENDCAPS_BOTHINVSRCALPHA;

	

	fill_src_blend_lut(GR_BLEND_ZERO,					D3DBLEND_ZERO,			0,						0, 0);
	fill_src_blend_lut(GR_BLEND_ONE,					D3DBLEND_ONE,			0,						0, 0);
//	fill_src_blend_lut(GR_BLEND_SRC_COLOR,				D3DBLEND_SRCCOLOR,		D3DBLEND_DESTCOLOR,		0, 0);
	fill_src_blend_lut(GR_BLEND_SRC_COLOR,				D3DBLEND_DESTCOLOR,		0,						0, 0);
	fill_src_blend_lut(GR_BLEND_ONE_MINUS_SRC_COLOR,	D3DBLEND_INVSRCCOLOR,	D3DBLEND_INVDESTCOLOR,	0, 0);
	fill_src_blend_lut(GR_BLEND_SRC_ALPHA,				D3DBLEND_SRCALPHA,		0,						0, 0);
	fill_src_blend_lut(GR_BLEND_ONE_MINUS_SRC_ALPHA,	D3DBLEND_INVSRCALPHA,	0,						0, 0);
	fill_src_blend_lut(GR_BLEND_DST_ALPHA,				D3DBLEND_DESTALPHA,		D3DBLEND_SRCALPHA,		0, 0);
	fill_src_blend_lut(GR_BLEND_ONE_MINUS_DST_ALPHA,	D3DBLEND_INVDESTALPHA,	D3DBLEND_INVSRCALPHA,	0, 0);
	fill_src_blend_lut(GR_BLEND_ALPHA_SATURATE,			D3DBLEND_SRCALPHASAT,	D3DBLEND_SRCALPHA,		0, 0);


	fill_dst_blend_lut(GR_BLEND_ZERO,					D3DBLEND_ZERO,			0,						0, 0);
	fill_dst_blend_lut(GR_BLEND_ONE,					D3DBLEND_ONE,			0,						0, 0);
	fill_dst_blend_lut(GR_BLEND_DST_COLOR,				D3DBLEND_DESTCOLOR,		D3DBLEND_SRCCOLOR,		0, 0);
	fill_dst_blend_lut(GR_BLEND_ONE_MINUS_DST_COLOR,	D3DBLEND_INVDESTCOLOR,	D3DBLEND_INVSRCCOLOR,	0, 0);
	fill_dst_blend_lut(GR_BLEND_SRC_ALPHA,				D3DBLEND_SRCALPHA,		0,						0, 0);
	fill_dst_blend_lut(GR_BLEND_ONE_MINUS_SRC_ALPHA,	D3DBLEND_INVSRCALPHA,	0,						0, 0);
	fill_dst_blend_lut(GR_BLEND_DST_ALPHA,				D3DBLEND_DESTALPHA,		D3DBLEND_SRCALPHA,		0, 0);
	fill_dst_blend_lut(GR_BLEND_ONE_MINUS_DST_ALPHA,	D3DBLEND_INVDESTALPHA,	D3DBLEND_INVSRCALPHA,	0, 0);
	fill_dst_blend_lut(GR_BLEND_ALPHA_SATURATE,			D3DBLEND_SRCALPHASAT,	D3DBLEND_SRCALPHA,		0, 0);







	atest_lut = new FxU32[8];

	atest_lut[GR_CMP_NEVER] = D3DCMP_NEVER;
	atest_lut[GR_CMP_LESS] = D3DCMP_LESS;
	atest_lut[GR_CMP_EQUAL] = D3DCMP_EQUAL;
	atest_lut[GR_CMP_LEQUAL] = D3DCMP_LESSEQUAL;
	atest_lut[GR_CMP_GREATER] = D3DCMP_GREATER;
	atest_lut[GR_CMP_NOTEQUAL] = D3DCMP_NOTEQUAL;
	atest_lut[GR_CMP_GEQUAL] = D3DCMP_GREATEREQUAL;
	atest_lut[GR_CMP_ALWAYS] = D3DCMP_ALWAYS;

	
	atest_caps_lut = new FxU32[8];

	atest_caps_lut[GR_CMP_NEVER] = D3DPCMPCAPS_NEVER;
	atest_caps_lut[GR_CMP_LESS] = D3DPCMPCAPS_LESS;
	atest_caps_lut[GR_CMP_EQUAL] = D3DPCMPCAPS_EQUAL;
	atest_caps_lut[GR_CMP_LEQUAL] = D3DPCMPCAPS_LESSEQUAL;
	atest_caps_lut[GR_CMP_GREATER] = D3DPCMPCAPS_GREATER;
	atest_caps_lut[GR_CMP_NOTEQUAL] = D3DPCMPCAPS_NOTEQUAL;
	atest_caps_lut[GR_CMP_GEQUAL] = D3DPCMPCAPS_GREATEREQUAL;
	atest_caps_lut[GR_CMP_ALWAYS] = D3DPCMPCAPS_ALWAYS;



	vconv_lut[DEPTH_D][GR_ORIGIN_UPPER_LEFT][TEXTURE]			= vconv_D_UL_T;
	vconv_lut[DEPTH_W][GR_ORIGIN_UPPER_LEFT][TEXTURE]			= vconv_W_UL_T;
	vconv_lut[DEPTH_W_Z16][GR_ORIGIN_UPPER_LEFT][TEXTURE]		= vconv_W_Z16_UL_T;
	vconv_lut[DEPTH_W_Z24][GR_ORIGIN_UPPER_LEFT][TEXTURE]		= vconv_W_Z24_UL_T;
	vconv_lut[DEPTH_W_Z32][GR_ORIGIN_UPPER_LEFT][TEXTURE]		= vconv_W_Z32_UL_T;
	vconv_lut[DEPTH_Z][GR_ORIGIN_UPPER_LEFT][TEXTURE]			= vconv_Z_UL_T;

	vconv_lut[DEPTH_D][GR_ORIGIN_LOWER_LEFT][TEXTURE]			= vconv_D_LL_T;
	vconv_lut[DEPTH_W][GR_ORIGIN_LOWER_LEFT][TEXTURE]			= vconv_W_LL_T;	
	vconv_lut[DEPTH_W_Z16][GR_ORIGIN_LOWER_LEFT][TEXTURE]		= vconv_W_Z16_LL_T;	
	vconv_lut[DEPTH_W_Z24][GR_ORIGIN_LOWER_LEFT][TEXTURE]		= vconv_W_Z24_LL_T;	
	vconv_lut[DEPTH_W_Z32][GR_ORIGIN_LOWER_LEFT][TEXTURE]		= vconv_W_Z32_LL_T;	
	vconv_lut[DEPTH_Z][GR_ORIGIN_LOWER_LEFT][TEXTURE]			= vconv_Z_LL_T;	
	
	vconv_lut[DEPTH_D][GR_ORIGIN_UPPER_LEFT][NO_TEXTURE]		= vconv_D_UL_NT;
	vconv_lut[DEPTH_W][GR_ORIGIN_UPPER_LEFT][NO_TEXTURE]		= vconv_W_UL_NT;
	vconv_lut[DEPTH_W_Z16][GR_ORIGIN_UPPER_LEFT][NO_TEXTURE]	= vconv_W_Z16_UL_NT;
	vconv_lut[DEPTH_W_Z24][GR_ORIGIN_UPPER_LEFT][NO_TEXTURE]	= vconv_W_Z24_UL_NT;
	vconv_lut[DEPTH_W_Z32][GR_ORIGIN_UPPER_LEFT][NO_TEXTURE]	= vconv_W_Z32_UL_NT;
	vconv_lut[DEPTH_Z][GR_ORIGIN_UPPER_LEFT][NO_TEXTURE]		= vconv_Z_UL_NT;

	vconv_lut[DEPTH_D][GR_ORIGIN_LOWER_LEFT][NO_TEXTURE]		= vconv_D_LL_NT;
	vconv_lut[DEPTH_W][GR_ORIGIN_LOWER_LEFT][NO_TEXTURE]		= vconv_W_LL_NT;
	vconv_lut[DEPTH_W_Z16][GR_ORIGIN_LOWER_LEFT][NO_TEXTURE]	= vconv_W_Z16_LL_NT;
	vconv_lut[DEPTH_W_Z24][GR_ORIGIN_LOWER_LEFT][NO_TEXTURE]	= vconv_W_Z24_LL_NT;
	vconv_lut[DEPTH_W_Z32][GR_ORIGIN_LOWER_LEFT][NO_TEXTURE]	= vconv_W_Z32_LL_NT;
	vconv_lut[DEPTH_Z][GR_ORIGIN_LOWER_LEFT][NO_TEXTURE]		= vconv_Z_LL_NT;

	int yab;
	float r, g, b, y1, a1, b1;
	for (yab=0; yab<256; yab++) {
		y1 = ((yab >> 4) & 0xF) / 15.0f;
		a1 = ((yab >> 2) & 0x3) / 3.0f;
		b1 = (yab & 0x3) / 3.0f;

		r = y1 + 0.95f*a1 + 0.62f*b1;
		g = y1 - 0.28f*a1 + 0.64f*b1;
		b = y1 - 1.11f*a1 + 1.73f*b1;

		if (r > 1.0f) r = 1.0f;
		if (g > 1.0f) g = 1.0f;
		if (b > 1.0f) b = 1.0f;
		if (r < 0.0f) r = 0.0f;
		if (g < 0.0f) g = 0.0f;
		if (b < 0.0f) b = 0.0f;

		yiq_lut[yab] = (int(r * 31) << 11) | (int(g * 63) << 5) | (int(b * 31));
	}
}

