// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"

/* Only for Marble32 */
FX_ENTRY void FX_CALL grSstOrigin( GrOriginLocation_t origin ) {
	vlog("Called grSstOrigin");

	Voodoo.origin = origin;
	vconv = vconv_lut[depthbuf_lut[Voodoo.dbMode]][Voodoo.origin][Voodoo.texture];
}

/* Only for Marble32 */
FX_ENTRY void FX_CALL grCullMode( GrCullMode_t mode ) {
	vlog("Called grCullMode: mode=" << mode);
	int dwCull = GR_CULL_DISABLE;

	switch(mode) {
	case GR_CULL_DISABLE:
		dwCull = D3DCULL_NONE;
		break;
	case GR_CULL_NEGATIVE:
		dwCull = D3DCULL_CCW;
		break;
	case GR_CULL_POSITIVE:
		dwCull = D3DCULL_CW;
		break;
  }
  if(DX.D3DD->SetRenderState(D3DRENDERSTATE_CULLMODE, dwCull) != D3D_OK) log("  Error in grCullMode\n");

}


/* Only for Marble32 */
FX_ENTRY FxBool FX_CALL gu3dfGetInfo( const char *filename, Gu3dfInfo *info ) {
	vlog("Called gu3dfGetInfo: filename=" << filename);

	ifstream f;
	int i, small_lod=1, large_lod=1, w_aspect=1, h_aspect=1;
	char s[256], fmt[8];

	f.open(filename);
	if(!f.is_open()) {
		log("  Could not open " << filename);
		return FXFALSE;
	}

	for (i=0; i<11; i++) {
		f >> s;
		switch (i) {
		case 2:
			strcpy(fmt, s);
			break;
		case 5:
			small_lod = atoi(s);
			break;
		case 6:
			large_lod = atoi(s);
			break;
		case 9:
			w_aspect = atoi(s);
			break;
		case 10:
			h_aspect = atoi(s);
			break;
		}
	}
	f.close();

	int slod, llod;

	switch (large_lod) {
	case 256:	llod = GR_LOD_256;	break;
	case 128:	llod = GR_LOD_128;	break;
	case 64:	llod = GR_LOD_64;	break;
	case 32:	llod = GR_LOD_32;	break;
	case 16:	llod = GR_LOD_16;	break;
	case 8:		llod = GR_LOD_8;	break;
	case 4:		llod = GR_LOD_4;	break;
	case 2:		llod = GR_LOD_2;	break;
	case 1:		llod = GR_LOD_1;	break;
	default:	llod = GR_LOD_1;
	}

	switch (small_lod) {
	case 256:	slod = GR_LOD_256;	break;
	case 128:	slod = GR_LOD_128;	break;
	case 64:	slod = GR_LOD_64;	break;
	case 32:	slod = GR_LOD_32;	break;
	case 16:	slod = GR_LOD_16;	break;
	case 8:		slod = GR_LOD_8;	break;
	case 4:		slod = GR_LOD_4;	break;
	case 2:		slod = GR_LOD_2;	break;
	case 1:		slod = GR_LOD_1;	break;
	default:	slod = GR_LOD_1;
	}

	int width = large_lod, height = large_lod;

	info->header.aspect_ratio = getAspectRatio(w_aspect, h_aspect);
	switch (info->header.aspect_ratio) {
	case GR_ASPECT_8x1:		height >>= 3;	break;
	case GR_ASPECT_4x1:		height >>= 2;	break;
	case GR_ASPECT_2x1:		height >>= 1;	break;
	case GR_ASPECT_1x1:		break;
	case GR_ASPECT_1x2:		width >>= 1;	break;
	case GR_ASPECT_1x4:		width >>= 2;	break;
	case GR_ASPECT_1x8:		width >>= 3;	break;
	}


	info->header.width =  width;
	info->header.height = height;
	info->header.small_lod = slod;
	info->header.large_lod = llod;
	info->header.format = getTextureFormat(fmt);

//	info->table.nccTable
//	info->table.palette

	info->data = 0;

	info->mem_required = getMemRequired(info, w_aspect, h_aspect);


	vlog("  small_lod=" << small_lod);
	vlog("  large_lod=" << large_lod);
	vlog("  w_aspect=" << w_aspect);
	vlog("  h_aspect=" << h_aspect);
	vlog("  width=" << info->header.width);
	vlog("  height=" << info->header.height);
	vlog("  mem_required=" << info->mem_required);


//	return FXFALSE;
	return FXTRUE;
}

/* Only for Marble32 */
FX_ENTRY FxBool FX_CALL gu3dfLoad( const char *filename, Gu3dfInfo *info ) {
	vlog("Called gu3dfLoad");

//	return FXFALSE;
	return FXTRUE;
}


/* Not important */
FX_ENTRY void FX_CALL grDepthBiasLevel( FxI16 level ) {
	vlog("Called grDepthBiasLevel: level=" << level);

	_exebuffer->Flush();

//	log("  Error, unsupported function grDepthBiasLevel");
//	int bias = 16 - level;
	int bias = level;
	
	if (level < 0) level = 0;
	if (level > 16) level = 16;

	DX.D3DD->SetRenderState(D3DRENDERSTATE_ZBIAS, level);

//	_depthBias = level;
}

/* Maybe important */
FX_ENTRY void FX_CALL grDepthMask( FxBool mask ) {
	vlog("Called grDepthMask: mask=" << mask);

	_exebuffer->Flush();

	if (mask) 
		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZWRITEENABLE, TRUE);
	else
		DX.D3DD->SetRenderState(D3DRENDERSTATE_ZWRITEENABLE, FALSE);
		
}

GrTextureFormat_t getTextureFormat(char *format) {
	if (strcmp(format, "yiq") == 0)			return GR_TEXFMT_YIQ_422;
	if (strcmp(format, "rgb565") == 0)		return GR_TEXFMT_RGB_565;
	if (strcmp(format, "a8") == 0)			return GR_TEXFMT_ALPHA_8;
	if (strcmp(format, "rgb332") == 0)		return GR_TEXFMT_RGB_332;
	if (strcmp(format, "p8") == 0)			return GR_TEXFMT_P_8;
	if (strcmp(format, "ayiq8422") == 0)	return GR_TEXFMT_AYIQ_8422;
//	if (strcmp(format, "argb8888") == 0)	return GR_TEXFMT_ARGB_8888;
	if (strcmp(format, "argb8332") == 0)	return GR_TEXFMT_ARGB_8332;
	if (strcmp(format, "argb4444") == 0)	return GR_TEXFMT_ARGB_4444;
	if (strcmp(format, "argb1555") == 0)	return GR_TEXFMT_ARGB_1555;
	if (strcmp(format, "ap88") == 0)		return GR_TEXFMT_AP_88;

	return 0;
}

GrAspectRatio_t getAspectRatio(FxU32 w, FxU32 h) {
	if (w == 1) {
		if (h == 1) return GR_ASPECT_1x1;
		if (h == 2) return GR_ASPECT_1x2;
		if (h == 4) return GR_ASPECT_1x4;
		if (h == 8) return GR_ASPECT_1x8;
	}
	if (w == 2) {
		return GR_ASPECT_2x1;
	}
	if (w == 4) {
		return GR_ASPECT_4x1;
	}
	if (w == 8) {
		return GR_ASPECT_8x1;
	}

	return GR_ASPECT_1x1;
}

FxU32 getMemRequired(Gu3dfInfo *info, FxU32 w_aspect, FxU32 h_aspect) {
	FxU32 pixels = 0;
	int i;

	for (i=info->header.small_lod; i<=info->header.large_lod; i *= 2)
		pixels += w_aspect * h_aspect * i * i;

	return (pixels * getBytesPerPixel(info->header.format));
}

FxU32 getBytesPerPixel(GrTextureFormat_t fmt) {
	switch (fmt) {
	case GR_TEXFMT_8BIT: return 1;
//	case GR_TEXFMT_RGB_332: return 1;
	case GR_TEXFMT_YIQ_422: return 1;
	case GR_TEXFMT_ALPHA_8: return 1;
	case GR_TEXFMT_INTENSITY_8: return 1;
	case GR_TEXFMT_ALPHA_INTENSITY_44: return 1;
	case GR_TEXFMT_P_8: return 1;
	case GR_TEXFMT_RSVD0: return 0;
	case GR_TEXFMT_RSVD1: return 1;
	case GR_TEXFMT_16BIT: return 2;
//	case GR_TEXFMT_ARGB_8332: return 2;			// Same as GR_TEXFMT_16BIT
	case GR_TEXFMT_AYIQ_8422: return 2;
	case GR_TEXFMT_RGB_565: return 2;
	case GR_TEXFMT_ARGB_1555: return 2;
	case GR_TEXFMT_ARGB_4444: return 2;
	case GR_TEXFMT_ALPHA_INTENSITY_88: return 2;
	case GR_TEXFMT_AP_88: return 2;
	case GR_TEXFMT_RSVD2: return 2;
	}

	return 2;
}

FxU32 getLfbWriteBytesPerPixel(GrLfbWriteMode_t fmt) {
	switch (fmt) {
	case GR_LFBWRITEMODE_565:			return 2;        
	case GR_LFBWRITEMODE_555:			return 2;
	case GR_LFBWRITEMODE_1555:			return 2;
	case GR_LFBWRITEMODE_RESERVED1:		return 0;
	case GR_LFBWRITEMODE_888:			return 3;
	case GR_LFBWRITEMODE_8888:			return 4;
	case GR_LFBWRITEMODE_RESERVED2:		return 0;
	case GR_LFBWRITEMODE_RESERVED3:		return 0;
	case GR_LFBWRITEMODE_RESERVED4:		return 0;
	case GR_LFBWRITEMODE_RESERVED5:		return 0;
	case GR_LFBWRITEMODE_RESERVED6:		return 0;
	case GR_LFBWRITEMODE_RESERVED7:		return 0;
	case GR_LFBWRITEMODE_565_DEPTH:		return 4;
	case GR_LFBWRITEMODE_555_DEPTH:		return 4;
	case GR_LFBWRITEMODE_1555_DEPTH:	return 4;
	case GR_LFBWRITEMODE_ZA16:			return 2;
	case GR_LFBWRITEMODE_ANY:			return 2;
	default:							return 2;
	}
}