// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"



void (*ckcolorconv)(unsigned char *, unsigned char *);
void (*texcolorconv)(unsigned char *, unsigned char *);


#define SetPixelFormatRGB565() \
	bpp = 2; \
	_paletted = 0; \
	DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB; \
	DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16; \
	DDSDesc.ddpfPixelFormat.dwRBitMask = 0xF800; \
	DDSDesc.ddpfPixelFormat.dwGBitMask = 0x07E0; \
	DDSDesc.ddpfPixelFormat.dwBBitMask = 0x001F; \
	DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x0000; \
	ckcolorconv = ARGB8888toRGB565;

#define SetPixelFormatARGB1555() \
	bpp = 2; \
	_paletted = 0; \
	DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS; \
	DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16; \
	DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x8000; \
	DDSDesc.ddpfPixelFormat.dwRBitMask = 0x7C00; \
	DDSDesc.ddpfPixelFormat.dwGBitMask = 0x03E0; \
	DDSDesc.ddpfPixelFormat.dwBBitMask = 0x001F; \
	ckcolorconv = ARGB8888toARGB1555;

#define SetPixelFormatARGB4444() \
	bpp = 2; \
	_paletted = 0; \
	DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS; \
	DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16; \
	DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0xF000; \
	DDSDesc.ddpfPixelFormat.dwRBitMask = 0x0F00; \
	DDSDesc.ddpfPixelFormat.dwGBitMask = 0x00F0; \
	DDSDesc.ddpfPixelFormat.dwBBitMask = 0x000F; \
	ckcolorconv = 0;

#define SetPixelFormatP8() \
	bpp = 1; \
	_paletted = 1; \
	DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_PALETTEINDEXED8; \
	DDSDesc.ddpfPixelFormat.dwRGBBitCount = 8; \
	DDSDesc.ddpfPixelFormat.dwRBitMask = 0x00; \
	DDSDesc.ddpfPixelFormat.dwGBitMask = 0x00; \
	DDSDesc.ddpfPixelFormat.dwBBitMask = 0x00; \
	DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x00; \
	ckcolorconv = 0; 


Tex::Tex(const FxU32 address, const GrTexInfo & texinfo) {
	DDSURFACEDESC2 DDSDesc;
	int width, height;
	int bpp, sbpp;				// Bytes per pixel, source bytes per pixel

	vlog("Called Tex::Tex");

	_palnum = -1;
	_base = address;
	_aspect = texinfo.aspectRatio;


	switch (texinfo.largeLod) {
	case GR_LOD_256:	width=256;	height=256;	break;
	case GR_LOD_128:	width=128;	height=128;	break;
	case GR_LOD_64:		width=64;	height=64;	break;
	case GR_LOD_32:		width=32;	height=32;	break;
	case GR_LOD_16:		width=16;	height=16;	break;
	case GR_LOD_8:		width=8;	height=8;	break;
	case GR_LOD_4:		width=4;	height=4;	break;
	case GR_LOD_2:		width=2;	height=2;	break;
	case GR_LOD_1:		width=1;	height=1;	break;
	default:
		log("  Error in large LOD Tex::Tex: " << texinfo.largeLod);
		width=1; height=1;
		break;
	}

	switch(texinfo.aspectRatio)	{
	case GR_ASPECT_8x1:		height >>= 3; 	break;
	case GR_ASPECT_4x1:		height >>= 2; 	break;
	case GR_ASPECT_2x1:		height >>= 1; 	break;
	case GR_ASPECT_1x1:		break;
	case GR_ASPECT_1x2:		width >>= 1; 	break;
	case GR_ASPECT_1x4:		width >>= 2; 	break;
	case GR_ASPECT_1x8:		width >>= 3; 	break;
	default:
		log("  Error in aspect ratio in Tex::Tex: " << _aspect);
		_aspect = GR_ASPECT_1x1;
		break;
	}

	memset(&DDSDesc, 0, sizeof(DDSURFACEDESC2));
	DDSDesc.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);

	switch (texinfo.format) {
	case GR_TEXFMT_INTENSITY_8:
		vlog("  pixel format: GR_TEXFMT_INTENSITY_8");
		bpp = 1;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_LUMINANCE;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitCount = 8;
		DDSDesc.ddpfPixelFormat.dwLuminanceBitMask = 0xFF;
		ckcolorconv = 0;
		texcolorconv = 0;
		break;
//	case GR_TEXFMT_8BIT:
	case GR_TEXFMT_RGB_332:
		vlog("  pixel format: GR_TEXFMT_RGB_332");
		sbpp = 1;
		SetPixelFormatRGB565();
		texcolorconv = RGB332toRGB565;
		break;
//	case GR_TEXFMT_YIQ_422:            
	case GR_TEXFMT_ALPHA_8:
		vlog("  pixel format: GR_TEXFMT_ALPHA_8");
		bpp = 1;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_ALPHA;
		DDSDesc.ddpfPixelFormat.dwAlphaBitDepth = 8;
		ckcolorconv = 0;
		texcolorconv = 0;
		break;
	case GR_TEXFMT_ALPHA_INTENSITY_44:
		vlog("  pixel format: GR_TEXFMT_ALPHA_INTENSITY_44");
		sbpp = 1;
		SetPixelFormatARGB4444();
		texcolorconv = AI44toARGB4444;
		break;
	case GR_TEXFMT_P_8: 
		vlog("  pixel format: GR_TEXFMT_P_8");
//		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_PALETTEINDEXED8;
		SetPixelFormatP8();
		texcolorconv = 0;
		break;
//	case GR_TEXFMT_16BIT:
	case GR_TEXFMT_ARGB_8332:
		vlog("  pixel format: GR_TEXFMT_ARGB_8332");
		bpp = 2;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 16;
		DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0xFF00;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0x00E0;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x001C;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x0003;
		ckcolorconv = ARGB8888toARGB8332;
		texcolorconv = 0;
		break;
//	case GR_TEXFMT_AYIQ_8422:
	case GR_TEXFMT_RGB_565:
		vlog("  pixel format: GR_TEXFMT_RGB_565");
		SetPixelFormatRGB565();
		texcolorconv = 0;
		break;
	case GR_TEXFMT_ARGB_1555:
		vlog("  pixel format: GR_TEXFMT_ARGB_1555");
		SetPixelFormatARGB1555();
		texcolorconv = 0;
		break;
	case GR_TEXFMT_ARGB_4444:
		vlog("  pixel format: GR_TEXFMT_ARGB_4444");
		SetPixelFormatARGB4444();
		texcolorconv = 0;
		break;
	case GR_TEXFMT_ALPHA_INTENSITY_88:
		vlog("  pixel format: GR_TEXFMT_ALPHA_INTENSITY_88");

		bpp = 32;
		sbpp = 16;
		_paletted = 0;
		DDSDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS;
		DDSDesc.ddpfPixelFormat.dwRGBBitCount = 32;
		DDSDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0xFF000000;
		DDSDesc.ddpfPixelFormat.dwRBitMask = 0x00FF0000;
		DDSDesc.ddpfPixelFormat.dwGBitMask = 0x0000FF00;
		DDSDesc.ddpfPixelFormat.dwBBitMask = 0x000000FF;
		ckcolorconv = 0;
		texcolorconv = AI88toARGB8888;

		break;
//	case GR_TEXFMT_AP_88:  
	case GR_TEXFMT_YIQ_422:
		vlog("  pixel format: GR_TEXFMT_YIQ_422");

		sbpp = 1;
		SetPixelFormatRGB565();
		texcolorconv = YIQ422toRGB565;

		break;
	default:
		log("  Unknown texture format " << texinfo.format << " in Tex::Tex");
		break;
	}

	DDSDesc.dwSize = sizeof(DDSURFACEDESC2);
	DDSDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;
//	DDSDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT | DDSD_MIPMAPCOUNT;
	DDSDesc.dwMipMapCount = 5; 
	DDSDesc.ddsCaps.dwCaps = DDSCAPS_3DDEVICE | DDSCAPS_TEXTURE;
//	DDSDesc.ddsCaps.dwCaps = DDSCAPS_3DDEVICE | DDSCAPS_TEXTURE | DDSCAPS_VIDEOMEMORY;
//	DDSDesc.ddsCaps.dwCaps = DDSCAPS_3DDEVICE | DDSCAPS_TEXTURE | DDSCAPS_MIPMAP | DDSCAPS_COMPLEX;
	DDSDesc.ddsCaps.dwCaps2 = DDSCAPS2_TEXTUREMANAGE;
	DDSDesc.dwWidth = _texwidth = width;
	DDSDesc.dwHeight = _texheight = height;
	DDSDesc.dwTextureStage = 0;
	DDSDesc.lPitch = width * bpp;
	DDSDesc.dwMipMapCount = texinfo.largeLod - texinfo.smallLod + 1;
//	DDSDesc.dwMipMapCount = 1;

	HRESULT href;

	href = DX.DD->CreateSurface(&DDSDesc, &_texDDS, 0);
	if (href == DD_OK) {
		vlog("  Created surface:");
		vlog("   width=" << width);
		vlog("   height=" << height);
	} else {
		_tex = 0;
		log("Error code " << (href&0xFF) << " in Tex::Tex on CreateSurface, line " << __LINE__ << ", file " << __FILE__);
	}


	// Ok, now we copy the texture to the surface
	DDSURFACEDESC2 DDSD;
	RECT r = {0, 0, width, height};

	DDSD.dwSize = sizeof(DDSURFACEDESC2);

	if (_texDDS->Lock(&r, &DDSD, DDLOCK_WAIT, 0) == DD_OK) {

		vlog("   width=" << DDSD.dwWidth);
		vlog("   height=" << DDSD.dwHeight);
		vlog("   pitch=" << DDSD.lPitch);

		if (texcolorconv) {
			for (unsigned int i=0; i<DDSD.dwHeight; i++) {
				for (unsigned int j=0; j<DDSD.dwWidth; j++) {
					texcolorconv((unsigned char *)DDSD.lpSurface+(i*DDSD.lPitch)+j*bpp, (unsigned char *)texinfo.data+((i*DDSD.dwWidth+j)*sbpp));
				}
			}

		} else {
			for (unsigned int i=0; i<DDSD.dwHeight; i++) {
				memcpy((char *)DDSD.lpSurface+(i*DDSD.lPitch), (char *)texinfo.data+(i*DDSD.dwWidth*bpp), DDSD.dwWidth*bpp);
			}
		}
		
		if (_texDDS->Unlock(&r) != DD_OK) log("Could not unlock surface on line " << __LINE__ << ", file " << __FILE__);

	} else {
		_tex = 0;
		log("Could not lock texture surface on line " << __LINE__);
		return;
	}

	if (_paletted) {
			LPDIRECTDRAWPALETTE DDP;

			if (DX.DD->CreatePalette(DDPCAPS_8BIT | DDPCAPS_ALLOW256 | DDPCAPS_INITIALIZE , Voodoo.palette, &DDP, 0) != DD_OK) {
				_tex = 0;
				log("Error creating palette");
			}
			if (_texDDS->SetPalette(DDP) != DD_OK) {
				_tex = 0;
				log("Error setting palette");
			}
			DDP->Release();
	}


	if (ckcolorconv) {
		DDCOLORKEY colorKey;
		DWORD c = 0;

		ckcolorconv((unsigned char *)&c, (unsigned char *)&Voodoo.colorKey);

		colorKey.dwColorSpaceLowValue = c;
		colorKey.dwColorSpaceHighValue = c;


		_texDDS->SetColorKey(DDCKEY_SRCBLT, &colorKey);
	}

	if (_texDDS->QueryInterface(IID_IDirect3DTexture2, (void**)&_tex) != D3D_OK) {
		_tex = 0;
		log("Error querying interface");
	}


}

Tex::~Tex() {
	vlog("Called Tex::~Tex");
	_tex->Release();
	_texDDS->Release();
}



void Tex::setpalette() {
	LPDIRECTDRAWPALETTE DDP;

	if (Voodoo.palnum != _palnum) {
		if (_texDDS->GetPalette(&DDP) != DD_OK) log("Error getting palette");
		if (DDP->SetEntries(0, 0, 256, Voodoo.palette) != DD_OK) log("Error setting palette");
		DDP->Release();
		_palnum = Voodoo.palnum;
	}
}

