// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"



FX_ENTRY void FX_CALL grTexDownloadTable( GrChipID_t tmu, GrTexTable_t type, void *data ) {
	vlog("Called grTexDownloadTable");

	int i;

	if (tmu) return;

	switch (type) {
	case GR_TEXTABLE_PALETTE:
		for (i=0; i<256; i++) {
			Voodoo.palette[i].peRed = (unsigned char)((((FxU32 *)data)[i] >> 16) & 0xFF);
			Voodoo.palette[i].peGreen = (unsigned char)((((FxU32 *)data)[i] >> 8) & 0xFF);
			Voodoo.palette[i].peBlue = (unsigned char)((((FxU32 *)data)[i]) & 0xFF);
			Voodoo.palette[i].peFlags = 0;
		}
		break;
	case GR_TEXTABLE_NCC0:
		for (i=0; i<256; i++) {
			Voodoo.palette[i].peRed = 0;
			Voodoo.palette[i].peGreen = 0;
			Voodoo.palette[i].peBlue = 0;
			Voodoo.palette[i].peFlags = 0;
		}
//		log("Error: NCC textures not supported.\n");
		break;
	case GR_TEXTABLE_NCC1:
		for (i=0; i<256; i++) {
			Voodoo.palette[i].peRed = 0;
			Voodoo.palette[i].peGreen = 0;
			Voodoo.palette[i].peBlue = 0;
			Voodoo.palette[i].peFlags = 0;
		}
//		log("Error: NCC textures not supported.\n");
		break;
	}

	Voodoo.palnum++;

	_exebuffer->ForceTex(Voodoo.curtex);
}

FX_ENTRY void FX_CALL grTexNCCTable( GrChipID_t tmu, GrNCCTable_t table ) {

}




FX_ENTRY FxU32 FX_CALL grTexCalcMemRequired(GrLOD_t lodmin, GrLOD_t lodmax, GrAspectRatio_t aspect, GrTextureFormat_t fmt) {
	vlog("Called grTexCalcMemRequired");

	return 1024;
}


FX_ENTRY FxU32 FX_CALL grTexTextureMemRequired( FxU32 evenOdd, GrTexInfo *info ) {
	vlog("Called grTexTextureMemRequired");

//	return 1;
	return 1024;
}


FX_ENTRY void FX_CALL grTexDownloadMipMap(GrChipID_t tmu, FxU32 startAddress, FxU32 evenOdd, GrTexInfo *info) {
	vlog("Called grTexDownloadMipMap");

	_texlist->AddTex(startAddress, *info);

}


FX_ENTRY void FX_CALL grTexDownloadMipMapLevel(GrChipID_t			tmu, 
											   FxU32				startAddress, 
											   GrLOD_t				thisLod, 
											   GrLOD_t				largeLod, 
											   GrAspectRatio_t		aspectRatio, 
											   GrTextureFormat_t	format, 
											   FxU32				evenOdd, 
											   void					*data ) {
	vlog("Called grTexDownloadMipMapLevel");

	GrTexInfo info;

	if (thisLod != largeLod) return;

	info.smallLod = thisLod;
	info.largeLod = thisLod;
	info.aspectRatio = aspectRatio;
	info.format = format;
	info.data = data;

	_texlist->AddTex(startAddress, info);

}

FX_ENTRY void FX_CALL grTexDownloadMipMapLevelPartial( GrChipID_t        tmu,
                                 FxU32             startAddress,
                                 GrLOD_t           thisLod,
                                 GrLOD_t           largeLod,
                                 GrAspectRatio_t   aspectRatio,
                                 GrTextureFormat_t format,
                                 FxU32             evenOdd,
                                 void              *data,
                                 int               start,
                                 int               end ) {
	vlog("Called grTexDownloadMipMapLevelPartial");

	log("  Error, unsupported function: grTexDownloadMipMapLevelPartial");

}

FX_ENTRY FxU32 FX_CALL grTexMinAddress( GrChipID_t tmu ) {
	vlog("Called grTexMinAddress: tmu=" << tmu << "  address=" << 0);

	return 0;
}


FX_ENTRY FxU32 FX_CALL grTexMaxAddress( GrChipID_t tmu ) {
	vlog("Called grTexMaxAddress: tmu=" << tmu << "  address=" << (MAX_TEXTURES-1));

	return (MAX_TEXTURES<<10)-1;
}


FX_ENTRY void FX_CALL grTexLodBiasValue(GrChipID_t tmu, float bias ) {
	vlog("Called grTexLocalBiasValue");
	vlog("  tmu=" << tmu);
	vlog("  bias=" << bias);

	log("  Error, unsupported function: grTexLodBiasValue");

}


FX_ENTRY void FX_CALL grTexClampMode(GrChipID_t tmu, GrTextureClampMode_t s_clampmode, GrTextureClampMode_t t_clampmode) {
	vvlog("Called grTexClampMode");
	vvlog("  tmu=" << tmu);
	vvlog("  s_clampmode=" << s_clampmode);
	vvlog("  t_clampmode=" << t_clampmode);

	_exebuffer->SetAddressMode(
		s_clampmode == GR_TEXTURECLAMP_WRAP ? D3DTADDRESS_WRAP : D3DTADDRESS_CLAMP,
		t_clampmode == GR_TEXTURECLAMP_WRAP ? D3DTADDRESS_WRAP : D3DTADDRESS_CLAMP);



}

FX_ENTRY void FX_CALL guTexCombineFunction(
                     GrChipID_t tmu,
                     GrTextureCombineFnc_t fnc
                     ) {
	vlog("Called guTexCombineFunction");
	vlog("  tmu=" << tmu);
	vlog("  func=" << fnc);

	switch (fnc) {
//	case GR_TEXTURECOMBINE_ZERO:    
	case GR_TEXTURECOMBINE_DECAL:    guColorCombineFunction(GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB); break;
/*	case GR_TEXTURECOMBINE_OTHER:      
	case GR_TEXTURECOMBINE_ADD: 
	case GR_TEXTURECOMBINE_MULTIPLY: 
	case GR_TEXTURECOMBINE_SUBTRACT:   
	case GR_TEXTURECOMBINE_DETAIL:
	case GR_TEXTURECOMBINE_DETAIL_OTHER:
	case GR_TEXTURECOMBINE_TRILINEAR_ODD:
	case GR_TEXTURECOMBINE_TRILINEAR_EVEN:
	case GR_TEXTURECOMBINE_ONE:
*/	default: log("  Error: unsupported guTexCombineFunction: " << fnc);
	}



//	log("  Error, unsupported function: guTexCombineFunction");

}

FX_ENTRY void FX_CALL grTexMipMapMode( GrChipID_t     tmu, 
                 GrMipMapMode_t mode,
                 FxBool         lodBlend ) {
	vvlog("Called grTexMipMapMode");
	vvlog("  tmu=" << tmu);
	vvlog("  mode=" << mode);
	vvlog("  blend=" << lodBlend);

}


FX_ENTRY void FX_CALL grTexFilterMode(
                GrChipID_t tmu,
                GrTextureFilterMode_t minfilter_mode,
                GrTextureFilterMode_t magfilter_mode
                ) {
	vvlog("Called grTexFilterMode:");
	vvlog("  tmu=" << tmu);
	vvlog("  minmode=" << minfilter_mode);
	vvlog("  magmode=" << magfilter_mode);

	_exebuffer->Flush();

//	_exebuffer->SetTexFilterMode(

	switch(minfilter_mode) {
	case GR_TEXTUREFILTER_POINT_SAMPLED:
		DX.D3DD->SetTextureStageState(tmu, D3DTSS_MINFILTER, D3DTFG_POINT);
		break;
    case GR_TEXTUREFILTER_BILINEAR:
		DX.D3DD->SetTextureStageState(tmu, D3DTSS_MINFILTER, D3DTFG_LINEAR);
//		DX.D3DD->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTFP_LINEAR);
		break;
	}

	switch(magfilter_mode) {
	case GR_TEXTUREFILTER_POINT_SAMPLED:
		DX.D3DD->SetTextureStageState(tmu, D3DTSS_MAGFILTER, D3DTFG_POINT);
		break;
    case GR_TEXTUREFILTER_BILINEAR:
		DX.D3DD->SetTextureStageState(tmu, D3DTSS_MAGFILTER, D3DTFG_LINEAR);
//		DX.D3DD->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTFP_LINEAR);
		break;
	}
}

FX_ENTRY void FX_CALL grTexDetailControl(GrChipID_t tmu, int lod_bias, FxU8 detail_scale, float detail_max) {
	log("  Error, unsupported function: grTexDetailControl");

}




FX_ENTRY void FX_CALL grTexSource( GrChipID_t tmu, FxU32 startAddress, FxU32 evenOdd, GrTexInfo *info ) {
	vvlog("Called grTexSource");

	_exebuffer->SetTex(_texlist->GetTex(startAddress, *info));
}


FX_ENTRY GrMipMapId_t FX_CALL guTexAllocateMemory( 	GrChipID_t tmu,
											FxU8 evenOddMask,
											int width, int height,
											GrTextureFormat_t format,
											GrMipMapMode_t mmMode,
											GrLOD_t smallLod, GrLOD_t largeLod,
											GrAspectRatio_t aspectRatio,
											GrTextureClampMode_t sClampMode,
											GrTextureClampMode_t tClampMode,
											GrTextureFilterMode_t minFilterMode,
											GrTextureFilterMode_t magFilterMode,
											float lodBias,
											FxBool lodBlend ) {
	vlog("Called guTexAllocateMemory");

	log("  Error, unsupported function: guTexAllocateMemory");

	return 0;
}

FX_ENTRY void FX_CALL guTexDownloadMipMap( GrMipMapId_t mmid, const void *src, const GuNccTable *nccTable ) {
	vlog("Called guTexDownloadMipMap");

	log("  Error, unsupported function: guTexDownloadMipMap");

}

FX_ENTRY void FX_CALL guTexSource( GrMipMapId_t mmid ) {
	vlog("Called guTexSource");

//	log("  Error, unsupported function guTexSource: mmid=" << mmid);

	if (*(int *)&mmid < 0) _exebuffer->SetTex(0);
}


FX_ENTRY void FX_CALL grTexCombine( GrChipID_t tmu,
							GrCombineFunction_t rgb_function,
							GrCombineFactor_t rgb_factor,
							GrCombineFunction_t alpha_function,
							GrCombineFactor_t alpha_factor,
							FxBool rgb_invert,
							FxBool alpha_invert ) {
	vlog("Called grTexCombine: (" << tmu << ", " << rgb_function << ", " << rgb_factor << ", " << alpha_function << ", " << alpha_factor << ", " << rgb_invert << ", " << alpha_invert << ")");

	_exebuffer->Flush();

/*
	switch (rgb_function) {
	case GR_COMBINE_FUNCTION_LOCAL:
		switch (rgb_factor) {
		case GR_COMBINE_FACTOR_ZERO:
			_lpD3DD->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
			break;
		}
		break;
	}

	switch (alpha_function) {
	case GR_COMBINE_FUNCTION_LOCAL:
		switch (rgb_factor) {
		case GR_COMBINE_FACTOR_ZERO:
			_lpD3DD->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			break;
		}
		break;
	}
*/
}


FX_ENTRY void FX_CALL grTexCombineFunction(GrChipID_t tmu, GrTextureCombineFnc_t fnc) {
	log("Called grTexCombineFunction: (" << tmu << ", " << fnc << ")");
	
}
