// Copyright (C) 1998-1999 Scott Cutler
// Please see the "readme.txt" file for license details

#include "include.h"



LPDIRECTDRAWSURFACE4 DDS;

LPDIRECTDRAWSURFACE4 getbuffer(GrBuffer_t buffer) {
	switch (buffer) {
	case GR_BUFFER_FRONTBUFFER:		return DX.DDSPrimary;
	case GR_BUFFER_BACKBUFFER:		return DX.DDSRender;
	case GR_BUFFER_AUXBUFFER:
	case GR_BUFFER_DEPTHBUFFER:
	case GR_BUFFER_ALPHABUFFER:
	case GR_BUFFER_TRIPLEBUFFER:
	default:						return DX.DDSRender;
	}
}


DDSURFACEDESC2 getlock(GrBuffer_t dst_buffer, GrLock_t type) {
	_exebuffer->Flush();

	DWORD dwFlags = DDLOCK_WAIT;
	DDSURFACEDESC2 DDSD;

	if (type == GR_LFB_READ_ONLY) dwFlags |= DDLOCK_READONLY;
	if (type == GR_LFB_WRITE_ONLY) dwFlags |= DDLOCK_WRITEONLY;

	DDS = getbuffer(dst_buffer);

	DDSD.dwSize = sizeof(DDSURFACEDESC2);

	if (DDS->Lock(0, &DDSD, DDLOCK_WRITEONLY|DDLOCK_WAIT, 0) != D3D_OK) {
		log("  Error locking surface in grLfbWriteRegion"); 
	}

	return DDSD;
}

int unlock() {
	if (DDS->Unlock(0) != D3D_OK) {
		log("  Error unlocking surface in grLfbReadRegion");
		return FXFALSE;
	}
	return FXTRUE;
}

/*
GR_LFBWRITEMODE_888        
GR_LFBWRITEMODE_8888       
GR_LFBWRITEMODE_565_DEPTH  
GR_LFBWRITEMODE_555_DEPTH  
GR_LFBWRITEMODE_1555_DEPTH 
GR_LFBWRITEMODE_ZA16       
GR_LFBWRITEMODE_ANY        
*/


GrLfbInfo_t lockinfo;
GrLfbWriteMode_t lockwritemode;


FX_ENTRY FxBool FX_CALL grLfbLock( GrLock_t type, GrBuffer_t buffer, GrLfbWriteMode_t writeMode,
           GrOriginLocation_t origin, FxBool pixelPipeline, GrLfbInfo_t *info) {

	unsigned int i, j;

	vlog("Called grLfbLock");
	vlog("  type=" << type);
	vlog("  buffer=" << buffer);
	vlog("  writemode=" << writeMode);
	vlog("  origin=" << origin);
	vlog("  pixelpipeline=" << pixelPipeline);

	if (writeMode == GR_LFBWRITEMODE_565) {
		DDSURFACEDESC2 DDSD = getlock(buffer, type);

		info->size = sizeof(GrLfbInfo_t);
		info->lfbPtr = (void *)DDSD.lpSurface;
		info->strideInBytes = DDSD.lPitch;
		info->writeMode = writeMode;
		info->origin = origin;

		lockinfo = *info;
	} else {
		DDSURFACEDESC2 DDSD = getlock(buffer, type);

		int dbpp = getLfbWriteBytesPerPixel(writeMode);
		int sbpp = 2;

		info->size = sizeof(GrLfbInfo_t);
		info->lfbPtr = (void *)new char[Voodoo.width*Voodoo.height*dbpp];
		info->strideInBytes = Voodoo.width*dbpp;
		info->writeMode = writeMode;
		info->origin = origin;

		lockinfo = *info;
		lockwritemode = writeMode;

		void (*bufcolorconv)(unsigned char *, unsigned char *);

		switch (writeMode) {
		case GR_LFBWRITEMODE_565:	bufcolorconv = RGB565toRGB565;		break;
		case GR_LFBWRITEMODE_555:	bufcolorconv = RGB565toRGB555;		break;
		case GR_LFBWRITEMODE_1555:	bufcolorconv = RGB565toARGB1555;	break;
		default:
			log("   Unsupported writeMode=" << writeMode << " in grLfbLock");
			break;
		}


		for (i=0; i<Voodoo.height; i++) {
			for (j=0; j<Voodoo.width; j++) {
//				*((uchar *)info->lfbPtr+(i*Voodoo.height+j)*dbpp) = 1;
//				*(((uchar *)info->lfbPtr+(i*Voodoo.height+j)*dbpp) + 1) = 1;
				bufcolorconv((uchar *)info->lfbPtr+(i*Voodoo.width+j)*dbpp, (uchar *)DDSD.lpSurface+(i*DDSD.lPitch+j*sbpp));
			}
		}

		unlock();
	}

	return FXTRUE;
}


FX_ENTRY FxBool FX_CALL grLfbUnlock( GrLock_t type, GrBuffer_t buffer ) {
	vlog("Called grLfbUnlock");

	unsigned int i, j;

	_exebuffer->Flush();

	if (lockinfo.writeMode == GR_LFBWRITEMODE_565) {
		LPDIRECTDRAWSURFACE4 DDS;

		DDS = getbuffer(buffer);

		DDS->Unlock(0);

	} else {
		DDSURFACEDESC2 DDSD = getlock(buffer, type);

		void (*bufcolorconv)(unsigned char *, unsigned char *);

		switch (lockwritemode) {
		case GR_LFBWRITEMODE_565:	bufcolorconv = RGB565toRGB565;		break;
		case GR_LFBWRITEMODE_555:	bufcolorconv = RGB555toRGB565;		break;
		case GR_LFBWRITEMODE_1555:	bufcolorconv = ARGB1555toRGB565;	break;
		default:
			log("   Unsupported lockwritemode=" << lockwritemode << " in grLfbUnlock");
			break;
		}

		int sbpp = getLfbWriteBytesPerPixel(lockwritemode);
		int dbpp = 2;

		for (i=0; i<Voodoo.height; i++) {
			for (j=0; j<Voodoo.width; j++) {
				bufcolorconv((uchar *)DDSD.lpSurface+(i*DDSD.lPitch+j*dbpp), (uchar *)lockinfo.lfbPtr+(i*Voodoo.width+j)*sbpp);
			}
		}

		delete [] lockinfo.lfbPtr;

		unlock();
	}

	return FXTRUE;
}

/*
GR_LFB_SRC_FMT_888       
GR_LFB_SRC_FMT_8888      
GR_LFB_SRC_FMT_565_DEPTH 
GR_LFB_SRC_FMT_555_DEPTH 
GR_LFB_SRC_FMT_1555_DEPTH
GR_LFB_SRC_FMT_ZA16      
GR_LFB_SRC_FMT_RLE16     
*/

FX_ENTRY FxBool FX_CALL grLfbWriteRegion( GrBuffer_t dst_buffer, FxU32 dst_x, FxU32 dst_y, GrLfbSrcFmt_t src_format, 
										 FxU32 src_width, FxU32 src_height, FxI32 src_stride, void *src_data ) {

//	log("(" << src_buffer << ", " << src_x << ", " << src_y << ", " << src_width << ", " << src_height << ", " << dst_stride << ")");

	DDSURFACEDESC2 DDSD = getlock(dst_buffer, GR_LFB_WRITE_ONLY);
	unsigned int i, j;

	void (*bufcolorconv)(unsigned char *, unsigned char *);

	switch (src_format) {
	case GR_LFB_SRC_FMT_565:	bufcolorconv = RGB565toRGB565;		break;
	case GR_LFB_SRC_FMT_555:	bufcolorconv = RGB555toRGB565;		break;
	case GR_LFB_SRC_FMT_1555:	bufcolorconv = ARGB1555toRGB565;	break;
	default:
		log("   Unsupported src_format=" << src_format << " in grLfbWriteRegion");
		break;
	}

	if (src_format == GR_LFB_SRC_FMT_565) {
		for (i=0; i<src_height; i++) {
			memcpy(&((char *)DDSD.lpSurface)[(i+dst_y)*DDSD.lPitch+dst_x], (uchar *)src_data+(i*src_stride), src_width*2);
		}
	} else {
		for (i=0; i<src_height; i++) {
			for (j=0; j<src_width; j++) {
				bufcolorconv((uchar *)DDSD.lpSurface+((i+dst_y)*DDSD.lPitch+(dst_x+j)*2), (uchar *)src_data+(i*src_stride)+j*2);
			}
		}
	}

	unlock();

	return FXTRUE;
}

FX_ENTRY FxBool FX_CALL grLfbReadRegion( GrBuffer_t src_buffer, FxU32 src_x, FxU32 src_y, FxU32 src_width, FxU32 src_height, FxU32 dst_stride, void *dst_data ) {

	vlog("  Called grLfbReadRegion");
//	log("(" << src_buffer << ", " << src_x << ", " << src_y << ", " << src_width << ", " << src_height << ", " << dst_stride << ")");

	_exebuffer->Flush();

	DDSURFACEDESC2 DDSD = getlock(src_buffer, GR_LFB_READ_ONLY);
	FxU32 i;
	char *dst_buf = (char *)dst_data;

	for (i=0; i<src_height; i++) {
//		memcpy(&dst_buf[i*dst_stride], &((char *)DDSD.lpSurface)[i*DDSD.lPitch], src_width*getBytesPerPixel(Voodoo.colorFormat));
		memcpy(&dst_buf[i*dst_stride], &((char *)DDSD.lpSurface)[(i+src_y)*DDSD.lPitch+src_x], src_width*getBytesPerPixel(Voodoo.colorFormat));
	}

	if (DDS->Unlock(0) != D3D_OK) {
		log("  Error unlocking surface in grLfbReadRegion");
		return FXFALSE;
	}

	return FXTRUE;
}
